@@ -5,6 +5,7 @@ import threading
 import traceback
 import os.path
 from datetime import datetime, timedelta
+from dateutil.parser import parse
 from time import sleep
 from optparse import make_option, OptionParser
 
@@ -16,7 +17,7 @@ from limpyd.exceptions import DoesNotExist
 from limpyd_jobs.version import __version__ as limpyd_jobs_version
 from limpyd_jobs import STATUSES, LimpydJobsException, ConfigurationException
 from limpyd_jobs.models import Queue, Job, Error
-from limpyd_jobs.utils import import_class, total_seconds
+from limpyd_jobs.utils import import_class, total_seconds, compute_delayed_until
 
 LOGGER_NAME = 'limpyd-jobs'
 logger = logging.getLogger(LOGGER_NAME)
@@ -403,7 +404,9 @@ class Worker(object):
                     job._cached_status = status
                     queue._cached_name = queue.name.hget()
 
-                    if status != STATUSES.WAITING:
+                    if status == STATUSES.DELAYED:
+                        self.job_delayed(job, queue)
+                    elif status != STATUSES.WAITING:
                         self.job_skipped(job, queue)
                     else:
                         try:
@@ -415,7 +418,13 @@ class Worker(object):
                                 trace = traceback.format_exc()
                             self.job_error(job, queue, e, trace)
                         else:
-                            self.job_success(job, queue, job_result)
+                            job._cached_status = job.status.hget()
+                            if job._cached_status == STATUSES.DELAYED:
+                                self.job_delayed(job, queue)
+                            elif job._cached_status == STATUSES.CANCELED:
+                                self.job_skipped(job, queue)
+                            else:
+                                self.job_success(job, queue, job_result)
                 except Exception, e:
                     self.log('[%s] unexpected error: %s' % (identifier, str(e)),
                                                                  level='error')
@@ -442,7 +451,7 @@ class Worker(object):
         """
         Called when an exception was raised during the execute call for a job.
         """
-        to_be_requeued = self.requeue_times and self.requeue_times >= int(job.tries.hget() or 0)
+        to_be_requeued = not job.must_be_cancelled_on_error and self.requeue_times and self.requeue_times >= int(job.tries.hget() or 0)
 
         if not to_be_requeued:
             job.queued.delete()
@@ -458,7 +467,7 @@ class Worker(object):
                                        trace=trace,
                                        **additional_fields)
 
-        self.log(self.job_error_message(job, queue, exception, trace), level='error')
+        self.log(self.job_error_message(job, queue, to_be_requeued, exception, trace), level='error')
 
         if hasattr(job, 'on_error'):
             job.on_error(queue, exception, trace)
@@ -486,14 +495,15 @@ class Worker(object):
 
         self.log(self.job_requeue_message(job, queue))
 
-    def job_error_message(self, job, queue, exception, trace=None):
+    def job_error_message(self, job, queue, to_be_requeued, exception, trace=None):
         """
         Return the message to log when a job raised an error
         """
-        return '[%s|%s|%s] error: %s' % (queue._cached_name,
-                                         job.pk.get(),
-                                         job._cached_identifier,
-                                         str(exception))
+        return '[%s|%s|%s] error: %s [%s]' % (queue._cached_name,
+                             job.pk.get(),
+                             job._cached_identifier,
+                             str(exception),
+                             'requeued' if to_be_requeued else 'NOT requeued')
 
     def job_requeue_message(self, job, queue):
         """
@@ -548,7 +558,8 @@ class Worker(object):
 
     def job_skipped(self, job, queue):
         """
-        Called if a job can't be run: canceled, already running or done.
+        Called if a job, before trying to run it, has not the "waiting" status,
+        or, after run, if its status was set to "canceled"
         """
         job.queued.delete()
         self.log(self.job_skipped_message(job, queue), level='warning')
@@ -557,8 +568,7 @@ class Worker(object):
 
     def job_skipped_message(self, job, queue):
         """
-        Return the message to log when a job can't be run: canceled, already
-        running or done
+        Return the message to log when a job was skipped
         """
         return '[%s|%s|%s] job skipped (current status: %s)' % (
                 queue._cached_name,
@@ -566,6 +576,45 @@ class Worker(object):
                 job._cached_identifier,
                 STATUSES.by_value(job._cached_status, 'UNKNOWN'))
 
+    def job_delayed(self, job, queue):
+        """
+        Called if a job, before trying to run it, has the "delayed" status, or,
+        after run, if its status was set to "delayed"
+        If delayed_until was not set, or is invalid, set it to 60sec in the future
+        """
+        delayed_until = job.delayed_until.hget()
+        if delayed_until:
+            try:
+                delayed_until = compute_delayed_until(delayed_until=parse(delayed_until))
+            except (ValueError, TypeError):
+                delayed_until = None
+
+        if not delayed_until:
+            # by default delay it for 60 seconds
+            delayed_until = compute_delayed_until(delayed_for=60)
+
+        job.enqueue_or_delay(
+            queue_name=queue._cached_name,
+            delayed_until=delayed_until,
+            queue_model=queue.__class__,
+        )
+
+        self.log(self.job_delayed_message(job, queue), level='warning')
+
+        if hasattr(job, 'on_delayed'):
+            job.on_delayed(queue)
+
+    def job_delayed_message(self, job, queue):
+        """
+        Return the message to log when a job was delayed just before or during
+        its execution
+        """
+        return '[%s|%s|%s] job delayed until %s' % (
+                queue._cached_name,
+                job.pk.get(),
+                job._cached_identifier,
+                job.delayed_until.hget())
+
 
 class WorkerConfig(object):
     """
